<?php

namespace MauticPlugin\MauticDisparoProBundle\Services;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Psr7\Response;
use Mautic\SmsBundle\Api\AbstractSmsApi;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;
use Mautic\CoreBundle\Helper\PhoneNumberHelper;
use Mautic\LeadBundle\Entity\Lead;
use Mautic\PageBundle\Model\TrackableModel;
use Mautic\PluginBundle\Helper\IntegrationHelper;
use Monolog\Logger;

class DisparoProApi extends AbstractSmsApi {

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * MessageBirdApi constructor.
     *
     * @param TrackableModel    $pageTrackableModel
     * @param PhoneNumberHelper $phoneNumberHelper
     * @param IntegrationHelper $integrationHelper
     * @param Logger            $logger
     *
     * @param Http              $http
     */
    public function __construct(TrackableModel $pageTrackableModel, PhoneNumberHelper $phoneNumberHelper, IntegrationHelper $integrationHelper, Logger $logger) {
        $this->logger = $logger;
        $this->integrationHelper = $integrationHelper;
        parent::__construct($pageTrackableModel);
    }

    /**
     * @param $number
     *
     * @return string
     *
     * @throws NumberParseException
     */
    protected function sanitizeNumber($number) {
        $util = PhoneNumberUtil::getInstance();
        $parsed = $util->parse($number, 'BR');
        return $util->format($parsed, PhoneNumberFormat::E164);
    }


    public function validateMobile($number, $prefix)
    {
        $phone = preg_replace("/[^0-9]/", "", $number);
        if (preg_match("/^(55[1-9]{2}[9]{0,1}[6-9]{1}[0-9]{7})$/", $phone)) {
            return $phone;
        }

        if (strlen($phone) == 11) {
            return $prefix.$phone;
        }

        return false;
    }

    /**
     * @param Lead   $contact
     * @param string $content
     *
     * @return bool|mixed|string
     */
    public function sendSms(Lead $contact, $content) {
        $number = $contact->getLeadPhoneNumber();
        $leadId = $contact->getId();

        if ($number === null) {
            return false;
        }

        $integration = $this->integrationHelper->getIntegrationObject('DisparoPro');
        if ($integration && $integration->getIntegrationSettings()->getIsPublished()) {
            $data = $integration->getDecryptedApiKeys();
            if (isset($data['auth_token'])) {
                $body = [
                    [
                        "numero" => $this->sanitizeNumber(
                            $this->validateMobile(
                                $number,
                                $data['prefix'])
                        ),
                        "servico" => $data['sms_type'],
                        "codificacao" => $data['codification_type'] == 2 ? 8 : 0,
                        "mensagem" => $content,
                        "parceiro_id" => $leadId,
                    ],
                ];
                try {
                    $headers = [
                        'Authorization' => 'Bearer ' . $data['auth_token'],
                        'Content-Type' => 'application/json',
                    ];

                    $client = new Client();
                    $response = $client->post(
                        'https://api.disparopro.com.br/mt?source=integrador', [
                            'headers' => $headers,
                            'body' => json_encode($body),
                        ]
                    );

                    return ($response->getStatusCode() == 200) ? true : false;
                } catch (ServerException $exception) {
                    $this->parseResponse($exception->getResponse(), $body);
                } catch (Exception $e) {
                    if (method_exists($e, 'getErrorMessage')) {
                        return $e->getErrorMessage();
                    } elseif (!empty($e->getMessage())) {
                        return $e->getMessage();
                    }

                    return false;
                }
            }
        }
    }

}
